@@ -1,10 +1,6 @@
 use serde::{Deserialize, Serialize};
-use tui_prompts::State as _;
 
-use crate::{
-    cmd_log::CmdLogEntry, items::TargetData, menu::Menu, prompt::PromptData, state::State,
-    term::Term, Res,
-};
+use crate::{items::TargetData, menu::Menu, state::State, term::Term, Res};
 use std::{fmt::Display, rc::Rc};
 
 pub(crate) mod checkout;
@@ -188,111 +184,10 @@ impl Display for Menu {
     }
 }
 
-pub(crate) fn create_y_n_prompt(mut action: Action, prompt: &'static str) -> Action {
-    let update_fn = Rc::new(move |state: &mut State, term: &mut Term| {
-        if state.prompt.state.status().is_pending() {
-            match state.prompt.state.value() {
-                "y" => {
-                    Rc::get_mut(&mut action).unwrap()(state, term)?;
-                    state.prompt.reset(term)?;
-                }
-                "" => (),
-                _ => {
-                    state
-                        .current_cmd_log
-                        .push(CmdLogEntry::Error("Aborted".to_string()));
-                    state.prompt.reset(term)?;
-                }
-            }
-        }
-        Ok(())
-    });
-
-    Rc::new(move |state: &mut State, _term: &mut Term| {
-        state.prompt.set(PromptData {
-            prompt_text: format!("{} (y or n)", prompt).into(),
-            update_fn: update_fn.clone(),
-        });
-
-        Ok(())
-    })
-}
-
-pub(crate) fn create_prompt(
-    prompt: &'static str,
-    on_success: fn(&mut State, &mut Term, &str) -> Res<()>,
-    hide_menu: bool,
-) -> Action {
-    create_prompt_with_default(prompt, on_success, |_| None, hide_menu)
-}
-
-pub(crate) fn create_prompt_with_default(
-    prompt: &'static str,
-    on_success: fn(&mut State, &mut Term, &str) -> Res<()>,
-    create_default_value: fn(&State) -> Option<String>,
-    hide_menu: bool,
-) -> Action {
-    Rc::new(move |state: &mut State, _term: &mut Term| {
-        set_prompt(
-            state,
-            prompt,
-            Box::new(on_success),
-            Box::new(create_default_value),
-            hide_menu,
-        );
-        Ok(())
-    })
-}
-
-type DefaultFn = Box<dyn Fn(&State) -> Option<String>>;
-type PromptAction = Box<dyn Fn(&mut State, &mut Term, &str) -> Res<()>>;
-
-pub(crate) fn set_prompt(
-    state: &mut State,
-    prompt: &'static str,
-    on_success: PromptAction,
-    default_fn: DefaultFn,
-    hide_menu: bool,
-) {
-    let prompt_text = if let Some(default) = default_fn(state) {
-        format!("{} (default {}):", prompt, default).into()
-    } else {
-        format!("{}:", prompt).into()
-    };
-
-    if hide_menu {
-        state.hide_menu();
-    }
-
-    state.prompt.set(PromptData {
-        prompt_text,
-        update_fn: Rc::new(move |state, term| {
-            if state.prompt.state.status().is_done() {
-                let input = state.prompt.state.value().to_string();
-                state.prompt.reset(term)?;
-
-                let default_value = default_fn(state);
-                let value = match (input.as_str(), &default_value) {
-                    ("", None) => "",
-                    ("", Some(selected)) => selected,
-                    (value, _) => value,
-                };
-
-                on_success(state, term, value)?;
-
-                if hide_menu {
-                    state.unhide_menu();
-                }
-            }
-            Ok(())
-        }),
-    });
+pub(crate) fn confirm(state: &mut State, term: &mut Term, prompt: &'static str) -> Res<()> {
+    state.confirm(term, prompt)
 }
 
 pub(crate) fn selected_rev(state: &State) -> Option<String> {
-    match &state.screen().get_selected_item().target_data {
-        Some(TargetData::Branch(branch)) => Some(branch.to_owned()),
-        Some(TargetData::Commit(commit)) => Some(commit.to_owned()),
-        _ => None,
-    }
+    state.selected_rev()
 }
