@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::io::Read;
 use std::io::Write;
 use std::ops::DerefMut;
@@ -7,17 +8,16 @@ use std::process::Stdio;
 use std::rc::Rc;
 use std::sync::Arc;
 use std::sync::RwLock;
+use std::time::Duration;
 
 use arboard::Clipboard;
 use crossterm::event;
 use crossterm::event::Event;
 use crossterm::event::KeyCode;
-use crossterm::event::KeyEventKind;
 use crossterm::event::KeyModifiers;
 use git2::Repository;
 use ratatui::layout::Size;
 use tui_prompts::State as _;
-use tui_prompts::Status;
 
 use crate::bindings::Bindings;
 use crate::cli;
@@ -25,16 +25,18 @@ use crate::cmd_log::CmdLog;
 use crate::cmd_log::CmdLogEntry;
 use crate::config::Config;
 use crate::error::Error;
+use crate::file_watcher::FileWatcher;
+use crate::items::TargetData;
 use crate::menu::Menu;
 use crate::menu::PendingMenu;
 use crate::ops::Op;
 use crate::prompt;
+use crate::prompt::PromptData;
 use crate::screen;
 use crate::screen::Screen;
 use crate::term::Term;
 use crate::ui;
 
-use super::GituEvent;
 use super::Res;
 
 pub(crate) struct State {
@@ -45,11 +47,13 @@ pub(crate) struct State {
     pub quit: bool,
     pub screens: Vec<Screen>,
     pub pending_menu: Option<PendingMenu>,
-    pub pending_cmd: Option<(Child, Arc<RwLock<CmdLogEntry>>)>,
+    pending_cmd: Option<(Child, Arc<RwLock<CmdLogEntry>>)>,
     enable_async_cmds: bool,
     pub current_cmd_log: CmdLog,
     pub prompt: prompt::Prompt,
     pub clipboard: Option<Clipboard>,
+    needs_redraw: bool,
+    file_watcher: Option<FileWatcher>,
 }
 
 impl State {
@@ -83,7 +87,7 @@ impl State {
             .inspect_err(|e| log::warn!("Couldn't initialize clipboard: {}", e))
             .ok();
 
-        Ok(Self {
+        let mut state = Self {
             repo,
             config,
             bindings,
@@ -96,76 +100,119 @@ impl State {
             current_cmd_log: CmdLog::new(),
             prompt: prompt::Prompt::new(),
             clipboard,
-        })
+            file_watcher: None,
+            needs_redraw: true,
+        };
+
+        state.file_watcher = state.init_file_watcher()?;
+        Ok(state)
     }
 
-    pub fn update(&mut self, term: &mut Term, events: &[GituEvent]) -> Res<()> {
-        for event in events {
-            log::debug!("{:?}", event);
+    fn init_file_watcher(&mut self) -> Res<Option<FileWatcher>> {
+        if !self.config.general.refresh_on_file_change.enabled {
+            return Ok(None);
+        }
 
-            match *event {
-                GituEvent::Term(Event::Resize(w, h)) => {
-                    for screen in self.screens.iter_mut() {
-                        screen.size = Size::new(w, h);
-                    }
-                }
-                GituEvent::Term(Event::Key(key)) => {
-                    if self.prompt.state.is_focused() {
-                        self.prompt.state.handle_key_event(key)
-                    } else if key.kind == KeyEventKind::Press {
-                        if self.pending_cmd.is_none() {
-                            self.current_cmd_log.clear();
-                        }
+        let hiding_untracked_files = !self
+            .repo
+            .config()
+            .map_err(Error::ReadGitConfig)?
+            .get_bool("status.showUntrackedFiles")
+            .unwrap_or(true);
 
-                        self.handle_key_input(term, key)?;
-                    }
-                }
-                GituEvent::FileUpdate => {
-                    self.screen_mut().update()?;
-                }
-                _ => (),
+        if hiding_untracked_files {
+            self.display_info("File watcher disabled (status.showUntrackedFiles is off)");
+
+            return Ok(None);
+        }
+
+        Ok(
+            FileWatcher::new(self.repo.workdir().expect("Bare repos unhandled"))
+                .inspect_err(|err| {
+                    self.display_error(err.to_string());
+                    self.display_info("File watcher disabled");
+                })
+                .ok(),
+        )
+    }
+
+    pub fn run(&mut self, term: &mut Term, max_tick_delay: Duration) -> Res<()> {
+        while !self.quit {
+            term.backend_mut().poll_event(max_tick_delay)?;
+            self.update(term)?;
+        }
+
+        Ok(())
+    }
+
+    pub fn update(&mut self, term: &mut Term) -> Res<()> {
+        if term.backend_mut().poll_event(Duration::ZERO)? {
+            let event = term.backend_mut().read_event()?;
+            self.handle_event(term, event)?;
+        }
+
+        if let Some(file_watcher) = &mut self.file_watcher {
+            if file_watcher.pending_updates() {
+                self.screen_mut().update()?;
+                self.stage_redraw();
             }
-
-            self.update_prompt(term)?;
         }
 
         let handle_pending_cmd_result = self.handle_pending_cmd();
-        let pending_cmd_done = self
-            .handle_result(handle_pending_cmd_result)
-            .unwrap_or(true);
+        self.handle_result(handle_pending_cmd_result)?;
 
-        let needs_redraw = !events.is_empty() || pending_cmd_done;
-
-        if needs_redraw && self.screens.last_mut().is_some() {
-            term.draw(|frame| ui::ui(frame, self))
-                .map_err(Error::Term)?;
+        if self.needs_redraw {
+            self.redraw_now(term)?;
         }
 
         Ok(())
     }
 
-    fn update_prompt(&mut self, term: &mut Term) -> Res<()> {
-        if self.prompt.state.status() == Status::Aborted {
-            self.unhide_menu();
-            self.prompt.reset(term)?;
-        } else if let Some(mut prompt_data) = self.prompt.data.take() {
-            let result = (Rc::get_mut(&mut prompt_data.update_fn).unwrap())(self, term);
+    pub fn handle_event(&mut self, term: &mut Term, event: Event) -> Res<()> {
+        log::debug!("{:?}", event);
 
-            match result {
-                Ok(()) => {
-                    if self.prompt.state.is_focused() {
-                        self.prompt.data = Some(prompt_data);
-                    }
+        match event {
+            Event::Resize(w, h) => {
+                for screen in self.screens.iter_mut() {
+                    screen.size = Size::new(w, h);
                 }
-                Err(error) => self
-                    .current_cmd_log
-                    .push(CmdLogEntry::Error(error.to_string())),
+
+                self.stage_redraw();
+                Ok(())
             }
+            Event::Key(key) => {
+                if self.pending_cmd.is_none() {
+                    self.current_cmd_log.clear();
+                }
+
+                if self.prompt.state.is_focused() {
+                    self.prompt.state.handle_key_event(key);
+                } else {
+                    self.handle_key_input(term, key)?;
+                }
+
+                self.stage_redraw();
+                Ok(())
+            }
+            _ => Ok(()),
         }
+    }
+
+    pub fn redraw_now(&mut self, term: &mut Term) -> Res<()> {
+        if self.screens.last_mut().is_some() {
+            term.draw(|frame| ui::ui(frame, self))
+                .map_err(Error::Term)?;
+
+            self.needs_redraw = false;
+        };
 
         Ok(())
     }
 
+    pub fn stage_redraw(&mut self) {
+        self.needs_redraw = true;
+    }
+
     fn handle_key_input(&mut self, term: &mut Term, key: event::KeyEvent) -> Res<()> {
         let menu = match &self.pending_menu {
             None => Menu::Root,
@@ -197,20 +244,22 @@ impl State {
         let target = self.screen().get_selected_item().target_data.as_ref();
         if let Some(mut action) = op.clone().implementation().get_action(target) {
             let result = Rc::get_mut(&mut action).unwrap()(self, term);
-            self.handle_result(result);
+            self.handle_result(result)?;
         }
 
         Ok(())
     }
 
-    fn handle_result<T>(&mut self, result: Res<T>) -> Option<T> {
+    fn handle_result<T>(&mut self, result: Res<T>) -> Res<()> {
         match result {
-            Ok(value) => Some(value),
+            Ok(_) => Ok(()),
+            Err(Error::NoMoreEvents) => Err(Error::NoMoreEvents),
+            Err(Error::PromptAborted) => Ok(()),
             Err(error) => {
                 self.current_cmd_log
                     .push(CmdLogEntry::Error(error.to_string()));
 
-                None
+                Ok(())
             }
         }
     }
@@ -228,13 +277,15 @@ impl State {
     }
 
     /// Displays an `Info` message to the CmdLog.
-    pub fn display_info(&mut self, message: String) {
-        self.current_cmd_log.push(CmdLogEntry::Info(message));
+    pub fn display_info<S: Into<Cow<'static, str>>>(&mut self, message: S) {
+        self.current_cmd_log
+            .push(CmdLogEntry::Info(message.into().into_owned()));
     }
 
     /// Displays an `Error` message to the CmdLog.
-    pub fn display_error(&mut self, error: String) {
-        self.current_cmd_log.push(CmdLogEntry::Error(error));
+    pub fn display_error<S: Into<Cow<'static, str>>>(&mut self, message: S) {
+        self.current_cmd_log
+            .push(CmdLogEntry::Error(message.into().into_owned()));
     }
 
     /// Runs a `Command` and handles its output.
@@ -290,13 +341,13 @@ impl State {
     }
 
     /// Handles any pending_cmd in State without blocking. Returns `true` if a cmd was handled.
-    pub fn handle_pending_cmd(&mut self) -> Res<bool> {
+    fn handle_pending_cmd(&mut self) -> Res<()> {
         let Some((ref mut child, ref mut log_rwlock)) = self.pending_cmd else {
-            return Ok(false);
+            return Ok(());
         };
 
         let Some(status) = child.try_wait().map_err(Error::CouldntAwaitCmd)? else {
-            return Ok(false);
+            return Ok(());
         };
 
         log::debug!("pending cmd finished with {:?}", status);
@@ -304,9 +355,10 @@ impl State {
         let result = write_child_output_to_log(log_rwlock, child, status);
         self.pending_cmd = None;
         self.screen_mut().update()?;
+        self.stage_redraw();
         result?;
 
-        Ok(true)
+        Ok(())
     }
 
     pub fn run_cmd_interactive(&mut self, term: &mut Term, mut cmd: Command) -> Res<()> {
@@ -317,7 +369,7 @@ impl State {
         cmd.current_dir(self.repo.workdir().ok_or(Error::NoRepoWorkdir)?);
 
         self.current_cmd_log.push_cmd_with_output(&cmd, "\n".into());
-        self.update(term, &[GituEvent::Refresh])?;
+        self.redraw_now(term)?;
 
         eprint!("\r");
 
@@ -394,6 +446,90 @@ impl State {
             menu.is_hidden = false;
         }
     }
+
+    pub fn selected_rev(&self) -> Option<String> {
+        match &self.screen().get_selected_item().target_data {
+            Some(TargetData::Branch(branch)) => Some(branch.to_owned()),
+            Some(TargetData::Commit(commit)) => Some(commit.to_owned()),
+            _ => None,
+        }
+    }
+
+    pub fn prompt(&mut self, term: &mut Term, params: &PromptParams) -> Res<String> {
+        let prompt_text = if let Some(default) = (params.create_default_value)(self) {
+            format!("{} (default {}):", params.prompt, default).into()
+        } else {
+            format!("{}:", params.prompt).into()
+        };
+
+        if params.hide_menu {
+            self.hide_menu();
+        }
+
+        self.prompt.set(PromptData { prompt_text });
+        self.redraw_now(term)?;
+
+        loop {
+            let event = term.backend_mut().read_event()?;
+            self.handle_event(term, event)?;
+
+            if self.prompt.state.status().is_done() {
+                let value = get_prompt_result(params, self)?;
+
+                self.unhide_menu();
+                self.prompt.reset(term)?;
+
+                return Ok(value);
+            } else if self.prompt.state.status().is_aborted() {
+                self.unhide_menu();
+                self.prompt.reset(term)?;
+
+                return Err(Error::PromptAborted);
+            }
+
+            self.redraw_now(term)?;
+        }
+    }
+
+    pub fn confirm(&mut self, term: &mut Term, prompt: &'static str) -> Res<()> {
+        self.hide_menu();
+        self.prompt.set(PromptData {
+            prompt_text: prompt.into(),
+        });
+        self.redraw_now(term)?;
+
+        loop {
+            let event = term.backend_mut().read_event()?;
+            self.handle_event(term, event)?;
+
+            match self.prompt.state.value() {
+                "y" => {
+                    self.prompt.reset(term)?;
+                    return Ok(());
+                }
+                "" => (),
+                _ => {
+                    self.prompt.reset(term)?;
+                    return Err(Error::PromptAborted);
+                }
+            }
+
+            self.redraw_now(term)?;
+        }
+    }
+}
+
+fn get_prompt_result(params: &PromptParams, state: &mut State) -> Res<String> {
+    let input = state.prompt.state.value();
+    let default_value = (params.create_default_value)(state);
+
+    let value = match (input, &default_value) {
+        ("", None) => "",
+        ("", Some(selected)) => selected,
+        (value, _) => value,
+    };
+
+    Ok(value.to_string())
 }
 
 fn tee(maybe_input: Option<&mut impl Read>, outputs: &mut [&mut dyn Write]) -> std::io::Result<()> {
@@ -465,3 +601,21 @@ fn write_child_output_to_log(
 
     Ok(())
 }
+
+type DefaultFn = Box<dyn Fn(&State) -> Option<String>>;
+
+pub(crate) struct PromptParams {
+    pub prompt: &'static str,
+    pub create_default_value: DefaultFn,
+    pub hide_menu: bool,
+}
+
+impl Default for PromptParams {
+    fn default() -> Self {
+        Self {
+            prompt: "",
+            create_default_value: Box::new(|_| None),
+            hide_menu: true,
+        }
+    }
+}
